@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -1313,6 +1313,13 @@ void cmd_handle(char *ostr) {
   // it is a command, extract cmd and args
   } else {
     cmd++;
+
+    if(strchr(cmd, '\n')) {
+      g_free(str);
+      ui_m(NULL, 0, "Commands cannot contain newlines");
+      return;
+    }
+
     char *sep = strchr(cmd, ' ');
     if(sep)
       *sep = 0;
