@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -281,7 +281,11 @@ static const doc_set_t doc_sets[] = {
   " separated with a comma. When unset, '0.0.0.0,::' is assumed. Only the IP"
   " version used to connect to the hub is used. That is, if you connect to an"
   " IPv6 hub, then the configured IPv6 address is used and the IPv4 address is"
-  " ignored."
+  " ignored.\n\n"
+  "When set to the special value `local', ncdc will automatically get your IP"
+  " address from the local network interface that is used to connect to the"
+  " hub. This option should only be used if there is no NAT between you and"
+  " the hub, because this will give the wrong IP if you are behind a NAT."
 },
 { "active_port", 1, "<integer>",
   "The listen port for incoming connections in active mode. Set to `0' to"
@@ -292,7 +296,7 @@ static const doc_set_t doc_sets[] = {
 },
 { "active_udp_port", 1, "<integer>",
   "The listen port for incoming UDP connections in active mode. Defaults to the"
-  " `active_port' setting, or to a random number of `active_port' is not set."
+  " `active_port' setting, or to a random number if `active_port' is not set."
 },
 { "adc_blom", 1, "<boolean>",
   "Whether to support the BLOM extension on ADC hubs. This may decrease the"
@@ -435,6 +439,14 @@ static const doc_set_t doc_sets[] = {
   " your system for other things besides ncdc, you share large files (>100MB)"
   " and people are not constantly downloading the same file from you."
 },
+{ "geoip_cc4", 0, "<path>|disabled",
+  "Path to the GeoIP Country database file for IPv4, or 'disabled' to disable"
+  " GeoIP lookup for IPv4 addresses."
+},
+{ "geoip_cc6", 0, "<path>|disabled",
+  "Path to the GeoIP Country database file for IPv6, or 'disabled' to disable"
+  " GeoIP lookup for IPv6 addresses."
+},
 { "hash_rate", 0, "<speed>",
   "Maximum file hashing speed. See the `download_rate' setting for allowed"
   " formats for this setting."
@@ -471,6 +483,10 @@ static const doc_set_t doc_sets[] = {
 { "log_downloads", 0, "<boolean>",
   "Log downloaded files to transfers.log."
 },
+{ "log_hubchat", 1, "<boolean>",
+  "Log the main hub chat. Note that changing this requires any affected hub"
+  " tabs to be closed and reopened before the change is effective."
+},
 { "log_uploads", 0, "<boolean>",
   "Log file uploads to transfers.log."
 },
