@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -50,7 +50,9 @@ GMainLoop *main_loop;
 #define INPT_CODE(key)  ((gunichar)((key)&G_GUINT64_CONSTANT(0xFFFFFFFF)))
 #define INPT_TYPE(key)  ((char)((key)>>32))
 
-#define KEY_ESCAPE (KEY_MAX+1)
+#define KEY_ESCAPE                (KEY_MAX+1)
+#define KEY_BRACKETED_PASTE_START (KEY_ESCAPE+1)
+#define KEY_BRACKETED_PASTE_END   (KEY_BRACKETED_PASTE_START+1)
 
 #endif
 
@@ -126,7 +128,7 @@ static void handle_input() {
       if(INPT_TYPE(key) != 1)
         continue;
       if(INPT_CODE(key) == '[') {
-        curignore = 0;
+        curignore = 1;
         continue;
       }
       key |= (guint64)3<<32; // a not very nice way of saying "turn this key into a INPT_ALT"
@@ -234,15 +236,15 @@ char *ncdc_version() {
 }
 
 
-static gboolean stderr_redir = FALSE;
+static FILE *stderrlog;
 
-// redirect all non-fatal errors to stderr (NOT stdout!)
+// redirect all non-fatal errors to the log
 static void log_redirect(const gchar *dom, GLogLevelFlags level, const gchar *msg, gpointer dat) {
-  if(!(level & (G_LOG_LEVEL_INFO|G_LOG_LEVEL_DEBUG)) || (stderr_redir && var_log_debug)) {
+  if(!(level & (G_LOG_LEVEL_INFO|G_LOG_LEVEL_DEBUG)) || (stderrlog != stderr && var_log_debug)) {
     char *ts = localtime_fmt("[%F %H:%M:%S %Z]");
-    fprintf(stderr, "%s *%s* %s\n", ts, loglevel_to_str(level), msg);
+    fprintf(stderrlog, "%s *%s* %s\n", ts, loglevel_to_str(level), msg);
     g_free(ts);
-    fflush(stderr);
+    fflush(stderrlog);
   }
 }
 
@@ -250,10 +252,10 @@ static void log_redirect(const gchar *dom, GLogLevelFlags level, const gchar *ms
 // clean-up our ncurses window before throwing a fatal error
 static void log_fatal(const gchar *dom, GLogLevelFlags level, const gchar *msg, gpointer dat) {
   endwin();
-  // print to both stderr (log file) and stdout
-  if(stderr_redir) {
-    fprintf(stderr, "\n\n*%s* %s\n", loglevel_to_str(level), msg);
-    fflush(stderr);
+  // print to both log file and stdout
+  if(stderrlog != stderr) {
+    fprintf(stderrlog, "\n\n*%s* %s\n", loglevel_to_str(level), msg);
+    fflush(stderrlog);
   }
   printf("\n\n*%s* %s\n", loglevel_to_str(level), msg);
 }
@@ -357,6 +359,7 @@ static gboolean print_version(const gchar *name, const gchar *val, gpointer dat,
 
 
 static gboolean auto_open = TRUE;
+static gboolean bracketed_paste = TRUE;
 
 static GOptionEntry cli_options[] = {
   { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
@@ -365,6 +368,8 @@ static GOptionEntry cli_options[] = {
       "Use a different session directory. Default: `$NCDC_DIR' or `$HOME/.ncdc'.", "<dir>" },
   { "no-autoconnect", 'n', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &auto_open,
       "Don't automatically connect to hubs with the `autoconnect' option set.", NULL },
+  { "no-bracketed-paste", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &bracketed_paste,
+      "Disable bracketed pasting.", NULL },
   { NULL }
 };
 
@@ -396,6 +401,8 @@ static void init_crypt() {
 
 int main(int argc, char **argv) {
   setlocale(LC_ALL, "");
+  // Early logging goes to stderr
+  stderrlog = stderr;
 
   // parse commandline options
   GOptionContext *optx = g_option_context_new("- NCurses Direct Connect");
@@ -430,14 +437,13 @@ int main(int argc, char **argv) {
   db_init();
   vars_init();
 
-  // redirect stderr to a log file
+  // open log file
   char *errlog = g_build_filename(db_dir, "stderr.log", NULL);
-  if(!freopen(errlog, "w", stderr)) {
+  if(!(stderrlog = fopen(errlog, "w"))) {
     fprintf(stderr, "ERROR: Couldn't open %s for writing: %s\n", errlog, strerror(errno));
     exit(1);
   }
   g_free(errlog);
-  stderr_redir = TRUE;
 
   // Init more stuff
   hub_init_global();
@@ -446,7 +452,9 @@ int main(int argc, char **argv) {
   cc_global_init();
   dl_init_global();
   ui_cmdhist_init("history");
-  ui_init();
+  ui_init(bracketed_paste);
+  geoip_reinit(4);
+  geoip_reinit(6);
 
   // setup SIGWINCH
   struct sigaction act;
@@ -502,6 +510,8 @@ int main(int argc, char **argv) {
     erase();
     refresh();
     endwin();
+    if(bracketed_paste)
+      ui_set_bracketed_paste(0);
 
     printf("Flushing unsaved data to disk...");
     fflush(stdout);
