@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -245,7 +245,22 @@ void ui_tab_incprio(ui_tab_t *tab, int prio) {
 }
 
 
-void ui_init() {
+// Based on gui_window_set_bracketed_paste_mode() in weechat's gui/curses/gui-curses-window.c
+void ui_set_bracketed_paste(int enable) {
+  char *env = getenv("TMUX");
+  int tmux = env && *env;
+  env = getenv("TERM");
+  int screen = env && (strncmp(env, "screen", 6) == 0) && !tmux;
+
+  fprintf(stderr, "%s\033[?2004%c%s",
+      screen ? "\033P" : "",
+      enable ? 'h' : 'l',
+      screen ? "\033\\" : "");
+  fflush(stderr);
+}
+
+
+void ui_init(gboolean bracketed_paste) {
   // init curses
   initscr();
   raw();
@@ -254,6 +269,13 @@ void ui_init() {
   keypad(stdscr, 1);
   nodelay(stdscr, 1);
 
+  // ensure curses is init'd before event-keys defined before events happen
+  if(bracketed_paste) {
+    define_key("\x1b[200~", KEY_BRACKETED_PASTE_START);
+    define_key("\x1b[201~", KEY_BRACKETED_PASTE_END);
+    ui_set_bracketed_paste(1);
+  }
+
   // global textinput field
   ui_global_textinput = ui_textinput_create(TRUE, cmd_suggest);
 
@@ -261,8 +283,6 @@ void ui_init() {
   ui_tab_open(uit_main_create(), TRUE, NULL);
 
   ui_colors_init();
-
-  // draw
   ui_draw();
 }
 
