@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -53,7 +53,7 @@ ui_tab_t *uit_main_create() {
 
 
 static void t_close(ui_tab_t *tab) {
-  ui_m(tab, 0, "Main tab cannot be closed.");
+  ui_m(NULL, UIM_NOLOG, "Type /quit to exit ncdc.");
 }
 
 
