@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2014 Yoran Heling
+  Copyright (c) 2011-2016 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -143,6 +143,12 @@ static char *p_ip(const char *val, GError **err) {
   return g_strdup_printf("%s,%s", ip4_unpack(i4), ip6_unpack(i6));
 }
 
+static char *p_active_ip(const char *val, GError **err) {
+  if(strcmp(val, "local") == 0)
+    return g_strdup(val);
+  return p_ip(val, err);
+}
+
 static char *p_regex(const char *val, GError **err) {
   GRegex *r = g_regex_new(val, 0, 0, err);
   if(!r)
@@ -603,6 +609,21 @@ static char *i_ffc() {
 }
 
 
+// geoip
+
+static gboolean s_geoip_cc(guint64 hub, const char *key, const char *val, GError **err) {
+#ifdef USE_GEOIP
+  int v = strcmp(key, "geoip_cc4") == 0 ? 4 : 6;
+  db_vars_set(hub, key, val);
+  geoip_reinit(v);
+  return TRUE;
+#else
+  g_set_error(err, 1, 0, "This option can't be modified: %s.", "Ncdc has not been compiled with GeoIP support");
+  return FALSE;
+#endif
+}
+
+
 // hubname
 
 static char *p_hubname(const char *val, GError **err) {
@@ -924,7 +945,7 @@ struct var_t {
 // name               g h  format          parse            suggest        getraw        setraw           default/init
 #define VARS\
   V(active,           1,1, f_bool,         p_bool,          su_bool,       NULL,         s_active_conf,   "false")\
-  V(active_ip,        1,1, f_id,           p_ip,            su_old,        NULL,         s_active_conf,   NULL)\
+  V(active_ip,        1,1, f_id,           p_active_ip,     su_old,        NULL,         s_active_conf,   NULL)\
   V(active_port,      1,1, f_int,          p_active_port,   NULL,          NULL,         s_active_conf,   NULL)\
   V(active_udp_port,  1,1, f_int,          p_active_port,   NULL,          g_active_udp, s_active_conf,   NULL)\
   V(adc_blom,         1,1, f_bool,         p_bool,          su_bool,       NULL,         NULL,            "false")\
@@ -947,6 +968,8 @@ struct var_t {
   V(filelist_maxage,  1,0, f_interval,     p_interval,      su_old,        NULL,         NULL,            "604800")\
   V(fl_done,          0,0, NULL,           NULL,            NULL,          NULL,         NULL,            "false")\
   V(flush_file_cache, 1,0, f_ffc,          p_ffc,           su_ffc,        g_ffc,        s_ffc,           i_ffc())\
+  V(geoip_cc4,        1,0, f_id,           p_id,            su_path,       NULL,         s_geoip_cc,      NULL)\
+  V(geoip_cc6,        1,0, f_id,           p_id,            su_path,       NULL,         s_geoip_cc,      NULL)\
   V(hash_rate,        1,0, f_speed,        p_speed,         NULL,          NULL,         NULL,            NULL)\
   V(hubaddr,          0,0, NULL,           NULL,            NULL,          NULL,         NULL,            NULL)\
   V(hubkp,            0,0, NULL,           NULL,            NULL,          NULL,         NULL,            NULL)\
@@ -955,6 +978,7 @@ struct var_t {
   V(local_address,    1,1, f_id,           p_ip,            su_old,        NULL,         s_active_conf,   db_vars_get(0, "active_bind"))\
   V(log_debug,        1,0, f_bool,         p_bool,          su_bool,       NULL,         s_log_debug,     i_log_debug())\
   V(log_downloads,    1,0, f_bool,         p_bool,          su_bool,       NULL,         NULL,            "true")\
+  V(log_hubchat,      1,1, f_bool,         p_bool,          su_bool,       NULL,         NULL,            "true")\
   V(log_uploads,      1,0, f_bool,         p_bool,          su_bool,       NULL,         NULL,            "true")\
   V(minislots,        1,0, f_int,          p_int_ge1,       NULL,          NULL,         NULL,            "3")\
   V(minislot_size,    1,0, f_minislot_size,p_minislot_size, NULL,          NULL,         NULL,            "65536")\
@@ -972,7 +996,7 @@ struct var_t {
   V(slots,            1,0, f_int,          p_int_ge1,       NULL,          NULL,         s_hubinfo,       "10")\
   V(sudp_policy,      1,0, f_sudp_policy,  p_sudp_policy,   su_sudp_policy,g_sudp_policy,s_sudp_policy,   G_STRINGIFY(VAR_SUDPP_PREFER))\
   V(tls_policy,       1,1, f_tls_policy,   p_tls_policy,    su_tls_policy, g_tls_policy, s_tls_policy,    G_STRINGIFY(VAR_TLSP_PREFER))\
-  V(tls_priority,     1,0, f_id,           p_tls_priority,  su_old,        NULL,         NULL,            "NORMAL")\
+  V(tls_priority,     1,0, f_id,           p_tls_priority,  su_old,        NULL,         NULL,            "NORMAL:-ARCFOUR-128")\
   V(ui_time_format,   1,0, f_id,           p_id,            su_old,        NULL,         NULL,            "[%H:%M:%S]")\
   V(upload_rate,      1,0, f_speed,        p_speed,         NULL,          NULL,         NULL,            NULL)
 
